@@ -30,7 +30,9 @@ private Q_SLOTS:
     void startProcessWithEnvironmentVariables();
 
     void doesNotFailOnNonExistingExecutable();
-    void shouldDoNothingOnEmptyCommand();
+    void shouldErrorOnEmptyCommand();
+
+    void runExecutableInLocalPath();
 };
 
 #endif /* COMMANDLAUNCHERJOBTEST_H */
