@@ -7,28 +7,31 @@
 
 #ifndef SCHEDULER_P_H
 #define SCHEDULER_P_H
+
+#include "worker_p.h"
+
 #include <QSet>
 #include <QTimer>
 // #define SCHEDULER_DEBUG
 
 namespace KIO
 {
-// The slave keeper manages the list of idle slaves that can be reused
-class SlaveKeeper : public QObject
+// The worker manager manages the list of idle workers that can be reused
+class WorkerManager : public QObject
 {
     Q_OBJECT
 public:
-    SlaveKeeper();
-    ~SlaveKeeper() override;
-    void returnSlave(KIO::Slave *slave);
-    // pick suitable slave for job and return it, return null if no slave found.
-    // the slave is removed from the keeper.
-    KIO::Slave *takeSlaveForJob(KIO::SimpleJob *job);
-    // remove slave from keeper
-    bool removeSlave(KIO::Slave *slave);
-    // remove all slaves from keeper
+    WorkerManager();
+    ~WorkerManager() override;
+    void returnWorker(KIO::Worker *worker);
+    // pick suitable worker for job and return it, return null if no worker found.
+    // the worker is removed from the manager.
+    KIO::Worker *takeWorkerForJob(KIO::SimpleJob *job);
+    // remove worker from manager
+    bool removeWorker(KIO::Worker *worker);
+    // remove all workers from manager
     void clear();
-    QList<KIO::Slave *> allSlaves() const;
+    QList<KIO::Worker *> allWorkers() const;
 
 private:
     void scheduleGrimReaper();
@@ -37,7 +40,7 @@ private Q_SLOTS:
     void grimReaper();
 
 private:
-    QMultiHash<QString, KIO::Slave *> m_idleSlaves;
+    QMultiHash<QString, KIO::Worker *> m_idleWorkers;
     QTimer m_grimTimer;
 };
 
@@ -73,55 +76,13 @@ public:
     KIO::SimpleJob *takeFirstInQueue();
     bool removeJob(KIO::SimpleJob *job);
 
-    QList<KIO::Slave *> allSlaves() const;
+    QList<KIO::Worker *> allWorkers() const;
 
 private:
     QMap<int, KIO::SimpleJob *> m_queuedJobs;
     QSet<KIO::SimpleJob *> m_runningJobs;
 };
 
-struct PerSlaveQueue {
-    PerSlaveQueue()
-        : runningJob(nullptr)
-    {
-    }
-    QList<SimpleJob *> waitingList;
-    SimpleJob *runningJob;
-};
-
-class ConnectedSlaveQueue : public QObject
-{
-    Q_OBJECT
-public:
-    ConnectedSlaveQueue();
-
-    bool queueJob(KIO::SimpleJob *job, KIO::Slave *slave);
-    bool removeJob(KIO::SimpleJob *job);
-
-    void addSlave(KIO::Slave *slave);
-    bool removeSlave(KIO::Slave *slave);
-
-    // KDE5: only one caller, for doubtful reasons. remove this if possible.
-    bool isIdle(KIO::Slave *slave);
-    bool isEmpty() const
-    {
-        return m_connectedSlaves.isEmpty();
-    }
-    QList<KIO::Slave *> allSlaves() const
-    {
-        return m_connectedSlaves.keys();
-    }
-
-private Q_SLOTS:
-    void startRunnableJobs();
-
-private:
-    // note that connected slaves stay here when idle, they are not returned to SlaveKeeper
-    QHash<KIO::Slave *, PerSlaveQueue> m_connectedSlaves;
-    QSet<KIO::Slave *> m_runnableSlaves;
-    QTimer m_startJobsTimer;
-};
-
 class SchedulerPrivate;
 
 class SerialPicker
@@ -137,8 +98,6 @@ public:
         return m_offset++;
     }
 
-    int changedPrioritySerial(int oldSerial, int newPriority) const;
-
 private:
     static const uint m_jobsPerPriority = 100000000;
     uint m_offset = 1;
@@ -155,12 +114,10 @@ public:
     ~ProtoQueue() override;
 
     void queueJob(KIO::SimpleJob *job);
-    void changeJobPriority(KIO::SimpleJob *job, int newPriority);
     void removeJob(KIO::SimpleJob *job);
-    KIO::Slave *createSlave(const QString &protocol, KIO::SimpleJob *job, const QUrl &url);
-    bool removeSlave(KIO::Slave *slave);
-    QList<KIO::Slave *> allSlaves() const;
-    ConnectedSlaveQueue m_connectedSlaveQueue; // KF6 TODO: remove
+    KIO::Worker *createWorker(const QString &protocol, KIO::SimpleJob *job, const QUrl &url);
+    bool removeWorker(KIO::Worker *worker);
+    QList<KIO::Worker *> allWorkers() const;
 
 private Q_SLOTS:
     // start max one (non-connected) job and return
@@ -171,7 +128,7 @@ private:
     QTimer m_startJobTimer;
     QMap<int, HostQueue *> m_queuesBySerial;
     QHash<QString, HostQueue> m_queuesByHostname;
-    SlaveKeeper m_slaveKeeper;
+    WorkerManager m_workerManager;
     int m_maxConnectionsPerHost;
     int m_maxConnectionsTotal;
     int m_runningJobsCount;
