@@ -311,6 +311,27 @@ public:
      */
     bool sortHiddenFoldersLast() const;
 
+    /**
+     * Puts \a widget to the right of the breadcrumb.
+     *
+     * KUrlNavigator takes ownership over \a widget. Any existing badge widget is deleted.
+     *
+     * NOTE: There is no limit to the size of the badge widget. If your badge widget is taller than other
+     * controls in KUrlNavigator, then the whole KUrlNavigator will be resized to accommodate it. Also,
+     * KUrlNavigator has fixed minimumWidth of 100, so if your badge widget is too wide, it might be clipped
+     * when the space is tight. You might want to call KUrlNavigator::setMinimumWidth() with a larger value
+     * in that case.
+     * In general, it is recommended to keep the badge widget small and not expanding, to avoid layout issues.
+     * @since 6.2
+     */
+    void setBadgeWidget(QWidget *widget);
+
+    /**
+     * Returns the badge widget set by setBadgeWidget(). If setBadgeWidget() hasn't been called, returns nullptr.
+     * @since 6.2
+     */
+    QWidget *badgeWidget() const;
+
 public Q_SLOTS:
     /**
      * Sets the location to \a url. The old URL is added to the history.
@@ -382,7 +403,8 @@ Q_SIGNALS:
     /**
      * Is emitted if the URL \a url should be opened in a new inactive tab because
      * the user clicked on a breadcrumb with the middle mouse button or
-     * left-clicked with the ctrl modifier pressed.
+     * left-clicked with the ctrl modifier pressed or pressed return with
+     * the alt modifier pressed.
      */
     void tabRequested(const QUrl &url);
 
@@ -390,14 +412,16 @@ Q_SIGNALS:
      * Is emitted if the URL \a url should be opened in a new active tab because
      * the user clicked on a breadcrumb with the middle mouse button with
      * the shift modifier pressed or left-clicked with both the ctrl and shift
-     * modifiers pressed.
+     * modifiers pressed or pressed return with both the alt and shift modifiers
+     * pressed.
      * @since 5.89
      */
     void activeTabRequested(const QUrl &url);
 
     /**
      * Is emitted if the URL \a url should be opened in a new window because
-     * the user left-clicked on a breadcrumb with the shift modifier pressed.
+     * the user left-clicked on a breadcrumb with the shift modifier pressed
+     * or pressed return with the shift modifier pressed.
      * @since 5.89
      */
     void newWindowRequested(const QUrl &url);
