@@ -17,10 +17,10 @@
 #include <QLineEdit>
 #include <QVBoxLayout>
 
-KIO::PasteDialog::PasteDialog(const QString &caption, const QString &label, const QString &value, const QStringList &items, QWidget *parent, bool clipboard)
+KIO::PasteDialog::PasteDialog(const QString &title, const QString &label, const QString &value, const QStringList &items, QWidget *parent)
     : QDialog(parent)
 {
-    setWindowTitle(caption);
+    setWindowTitle(title);
     setModal(true);
 
     QVBoxLayout *topLayout = new QVBoxLayout(this);
@@ -54,16 +54,6 @@ KIO::PasteDialog::PasteDialog(const QString &caption, const QString &label, cons
     topLayout->addWidget(buttonBox);
 
     setMinimumWidth(350);
-
-    m_clipboardChanged = false;
-    if (clipboard) {
-        connect(QApplication::clipboard(), &QClipboard::dataChanged, this, &PasteDialog::slotClipboardDataChanged);
-    }
-}
-
-void KIO::PasteDialog::slotClipboardDataChanged()
-{
-    m_clipboardChanged = true;
 }
 
 QString KIO::PasteDialog::lineEditText() const
@@ -75,3 +65,5 @@ int KIO::PasteDialog::comboItem() const
 {
     return m_comboBox->currentIndex();
 }
+
+#include "moc_pastedialog_p.cpp"
